@@ -27,16 +27,17 @@
  * dma buffers.  Use strip/fan hardware primitives where possible.
  * Simulate missing primitives with indexed vertices.
  */
-#include "glheader.h"
-#include "context.h"
-#include "macros.h"
-#include "imports.h"
-#include "mtypes.h"
+#include "main/glheader.h"
+#include "main/context.h"
+#include "main/macros.h"
+#include "main/imports.h"
+#include "main/mtypes.h"
+
+#include "math/m_xform.h"
 
 #include "tnl/t_context.h"
 
 #include "savagecontext.h"
-#include "savagetris.h"
 #include "savagestate.h"
 #include "savageioctl.h"
 
@@ -198,7 +199,7 @@ static GLboolean savage_run_render( GLcontext *ctx,
 
    for (i = 0 ; i < VB->PrimitiveCount ; i++)
    {
-      GLuint prim = VB->Primitive[i].mode;
+      GLuint prim = _tnl_translate_prim(&VB->Primitive[i]);
       GLuint start = VB->Primitive[i].start;
       GLuint length = VB->Primitive[i].count;
 
@@ -246,57 +247,59 @@ static GLboolean run_texnorm_stage( GLcontext *ctx,
       return GL_TRUE;
 
    for (i = 0 ; i < ctx->Const.MaxTextureUnits ; i++) {
-      if (!ctx->Texture.Unit[i]._ReallyEnabled ||
-	  VB->TexCoordPtr[i]->size == 4)
-	 /* Never try to normalize homogenous tex coords! */
-	 continue;
+      const GLbitfield reallyEnabled = ctx->Texture.Unit[i]._ReallyEnabled;
+      if (reallyEnabled) {
+         const struct gl_texture_object *texObj = ctx->Texture.Unit[i]._Current;
+         const GLboolean normalizeS = (texObj->WrapS == GL_REPEAT);
+         const GLboolean normalizeT = (reallyEnabled & TEXTURE_2D_BIT) &&
+            (texObj->WrapT == GL_REPEAT);
+         const GLfloat *in = (GLfloat *)VB->AttribPtr[_TNL_ATTRIB_TEX0 + i]->data;
+         const GLint instride = VB->AttribPtr[_TNL_ATTRIB_TEX0 + i]->stride;
+         GLfloat (*out)[4] = store->texcoord[i].data;
+         GLint j;
 
-      GLuint reallyEnabled = ctx->Texture.Unit[i]._ReallyEnabled;
-      struct gl_texture_object *texObj = ctx->Texture.Unit[i]._Current;
-      GLboolean normalizeS = (texObj->WrapS == GL_REPEAT);
-      GLboolean normalizeT = (reallyEnabled & TEXTURE_2D_BIT) &&
-	 (texObj->WrapT == GL_REPEAT);
-      GLfloat *in = (GLfloat *)VB->TexCoordPtr[i]->data;
-      GLint instride = VB->TexCoordPtr[i]->stride;
-      GLfloat (*out)[4] = store->texcoord[i].data;
-      GLint j;
+         if (!ctx->Texture.Unit[i]._ReallyEnabled ||
+             VB->AttribPtr[_TNL_ATTRIB_TEX0 + i]->size == 4)
+            /* Never try to normalize homogenous tex coords! */
+            continue;
 
-      if (normalizeS && normalizeT) {
-	 /* take first texcoords as rough estimate of mean value */
-	 GLfloat correctionS = -floor(in[0]+0.5);
-	 GLfloat correctionT = -floor(in[1]+0.5);
-	 for (j = 0; j < VB->Count; ++j) {
-	    out[j][0] = in[0] + correctionS;
-	    out[j][1] = in[1] + correctionT;
-	    in = (GLfloat *)((GLubyte *)in + instride);
-	 }
-      } else if (normalizeS) {
-	 /* take first texcoords as rough estimate of mean value */
-	 GLfloat correctionS = -floor(in[0]+0.5);
-	 if (reallyEnabled & TEXTURE_2D_BIT) {
-	    for (j = 0; j < VB->Count; ++j) {
-	       out[j][0] = in[0] + correctionS;
-	       out[j][1] = in[1];
-	       in = (GLfloat *)((GLubyte *)in + instride);
-	    }
-	 } else {
-	    for (j = 0; j < VB->Count; ++j) {
-	       out[j][0] = in[0] + correctionS;
-	       in = (GLfloat *)((GLubyte *)in + instride);
-	    }
-	 }
-      } else if (normalizeT) {
-	 /* take first texcoords as rough estimate of mean value */
-	 GLfloat correctionT = -floor(in[1]+0.5);
-	 for (j = 0; j < VB->Count; ++j) {
-	    out[j][0] = in[0];
-	    out[j][1] = in[1] + correctionT;
-	    in = (GLfloat *)((GLubyte *)in + instride);
-	 }
+         if (normalizeS && normalizeT) {
+            /* take first texcoords as rough estimate of mean value */
+            GLfloat correctionS = -floor(in[0]+0.5);
+            GLfloat correctionT = -floor(in[1]+0.5);
+            for (j = 0; j < VB->Count; ++j) {
+               out[j][0] = in[0] + correctionS;
+               out[j][1] = in[1] + correctionT;
+               in = (GLfloat *)((GLubyte *)in + instride);
+            }
+         } else if (normalizeS) {
+            /* take first texcoords as rough estimate of mean value */
+            GLfloat correctionS = -floor(in[0]+0.5);
+            if (reallyEnabled & TEXTURE_2D_BIT) {
+               for (j = 0; j < VB->Count; ++j) {
+                  out[j][0] = in[0] + correctionS;
+                  out[j][1] = in[1];
+                  in = (GLfloat *)((GLubyte *)in + instride);
+               }
+            } else {
+               for (j = 0; j < VB->Count; ++j) {
+                  out[j][0] = in[0] + correctionS;
+                  in = (GLfloat *)((GLubyte *)in + instride);
+               }
+            }
+         } else if (normalizeT) {
+            /* take first texcoords as rough estimate of mean value */
+            GLfloat correctionT = -floor(in[1]+0.5);
+            for (j = 0; j < VB->Count; ++j) {
+               out[j][0] = in[0];
+               out[j][1] = in[1] + correctionT;
+               in = (GLfloat *)((GLubyte *)in + instride);
+            }
+         }
+
+         if (normalizeS || normalizeT)
+            VB->AttribPtr[_TNL_ATTRIB_TEX0 + i] = &store->texcoord[i];
       }
-
-      if (normalizeS || normalizeT)
-	 VB->AttribPtr[VERT_ATTRIB_TEX0+i] = VB->TexCoordPtr[i] = &store->texcoord[i];
    }
 
    return GL_TRUE;
